@@ -6,7 +6,7 @@
  * Status:        Stable.
  * Author:        Dag Brattli <dagb@cs.uit.no>
  * Created at:    Sat Nov  7 21:43:15 1998
- * Modified at:   Tue Apr 25 21:19:12 2000
+ * Modified at:   Wed Mar  1 11:29:34 2000
  * Modified by:   Dag Brattli <dagb@cs.uit.no>
  * 
  *     Copyright (c) 1998-2000 Dag Brattli <dagb@cs.uit.no>
@@ -49,18 +49,15 @@
 #include <linux/netdevice.h>
 #include <linux/ioport.h>
 #include <linux/delay.h>
-#include <linux/malloc.h>
+#include <linux/slab.h>
 #include <linux/init.h>
 #include <linux/rtnetlink.h>
 
 #include <asm/io.h>
 #include <asm/dma.h>
 #include <asm/byteorder.h>
-#include <asm/hardirq.h>
 
-#ifdef CONFIG_APM
-#include <linux/apm_bios.h>
-#endif
+#include <linux/pm.h>
 
 #include <net/irda/wrapper.h>
 #include <net/irda/irda.h>
@@ -77,7 +74,7 @@ static char *driver_name = "nsc-ircc";
 
 /* Module parameters */
 static int qos_mtt_bits = 0x07;  /* 1 ms or more */
-static int dongle_id = 0;
+static int dongle_id;
 
 /* Use BIOS settions by default, but user may supply module parameters */
 static unsigned int io[]  = { ~0, ~0, ~0, ~0 };
@@ -93,7 +90,7 @@ static int nsc_ircc_init_338(nsc_chip_t *chip, chipio_t *info);
 static nsc_chip_t chips[] = {
 	{ "PC87108", { 0x150, 0x398, 0xea }, 0x05, 0x10, 0xf0, 
 	  nsc_ircc_probe_108, nsc_ircc_init_108 },
-	{ "PC87338", { 0x398, 0x15c, 0x2e }, 0x08, 0xb0, 0xf0, 
+	{ "PC87338", { 0x398, 0x15c, 0x2e }, 0x08, 0xb0, 0xf8, 
 	  nsc_ircc_probe_338, nsc_ircc_init_338 },
 	{ NULL }
 };
@@ -129,8 +126,8 @@ static int  nsc_ircc_setup(chipio_t *info);
 static void nsc_ircc_pio_receive(struct nsc_ircc_cb *self);
 static int  nsc_ircc_dma_receive(struct nsc_ircc_cb *self); 
 static int  nsc_ircc_dma_receive_complete(struct nsc_ircc_cb *self, int iobase);
-static int  nsc_ircc_hard_xmit_sir(struct sk_buff *skb, struct device *dev);
-static int  nsc_ircc_hard_xmit_fir(struct sk_buff *skb, struct device *dev);
+static int  nsc_ircc_hard_xmit_sir(struct sk_buff *skb, struct net_device *dev);
+static int  nsc_ircc_hard_xmit_fir(struct sk_buff *skb, struct net_device *dev);
 static int  nsc_ircc_pio_write(int iobase, __u8 *buf, int len, int fifo_size);
 static void nsc_ircc_dma_xmit(struct nsc_ircc_cb *self, int iobase);
 static void nsc_ircc_change_speed(struct nsc_ircc_cb *self, __u32 baud);
@@ -139,14 +136,12 @@ static int  nsc_ircc_is_receiving(struct nsc_ircc_cb *self);
 static int  nsc_ircc_read_dongle_id (int iobase);
 static void nsc_ircc_init_dongle_interface (int iobase, int dongle_id);
 
-static int  nsc_ircc_net_init(struct device *dev);
-static int  nsc_ircc_net_open(struct device *dev);
-static int  nsc_ircc_net_close(struct device *dev);
-static int  nsc_ircc_net_ioctl(struct device *dev, struct ifreq *rq, int cmd);
-static struct net_device_stats *nsc_ircc_net_get_stats(struct device *dev);
-#ifdef CONFIG_APM
-static int nsc_ircc_apmproc(apm_event_t event);
-#endif /* CONFIG_APM */
+static int  nsc_ircc_net_init(struct net_device *dev);
+static int  nsc_ircc_net_open(struct net_device *dev);
+static int  nsc_ircc_net_close(struct net_device *dev);
+static int  nsc_ircc_net_ioctl(struct net_device *dev, struct ifreq *rq, int cmd);
+static struct net_device_stats *nsc_ircc_net_get_stats(struct net_device *dev);
+static int nsc_ircc_pmproc(struct pm_dev *dev, pm_request_t rqst, void *data);
 
 /*
  * Function nsc_ircc_init ()
@@ -165,7 +160,7 @@ int __init nsc_ircc_init(void)
 	int i = 0;
 
 	/* Probe for all the NSC chipsets we know about */
-	for (chip=chips; chip->name ; chip++,i++) {
+	for (chip=chips; chip->name ; chip++) {
 		IRDA_DEBUG(2, __FUNCTION__"(), Probing for %s ...\n", 
 			   chip->name);
 		
@@ -201,7 +196,7 @@ int __init nsc_ircc_init(void)
 				 * we init the chip, if not we probe the values
 				 * set by the BIOS
 				 */				
-				if (io[i] < 2000) {
+				if (io[i] < 0x2000) {
 					chip->init(chip, &info);
 				} else
 					chip->probe(chip, &info);
@@ -216,11 +211,6 @@ int __init nsc_ircc_init(void)
 		} 
 		
 	}
-#ifdef CONFIG_APM
-	/* Make sure at least one chip was found before enabling APM */
-	if (ret == 0)
-		apm_register_callback(nsc_ircc_apmproc);
-#endif /* CONFIG_APM */
 
 	return ret;
 }
@@ -236,9 +226,7 @@ static void nsc_ircc_cleanup(void)
 {
 	int i;
 
-#ifdef CONFIG_APM
-	apm_unregister_callback(nsc_ircc_apmproc);
-#endif /* CONFIG_APM */
+	pm_unregister_all(nsc_ircc_pmproc);
 
 	for (i=0; i < 4; i++) {
 		if (dev_self[i])
@@ -255,16 +243,22 @@ static void nsc_ircc_cleanup(void)
  */
 static int nsc_ircc_open(int i, chipio_t *info)
 {
-	struct device *dev;
+	struct net_device *dev;
 	struct nsc_ircc_cb *self;
+        struct pm_dev *pmdev;
 	int ret;
 	int err;
 
 	IRDA_DEBUG(2, __FUNCTION__ "()\n");
 
+	MESSAGE("%s, Found chip at base=0x%03x\n", driver_name,
+		info->cfg_base);
+
 	if ((nsc_ircc_setup(info)) == -1)
 		return -1;
 
+	MESSAGE("%s, driver loaded (Dag Brattli)\n", driver_name);
+
 	/* Allocate new instance of the driver */
 	self = kmalloc(sizeof(struct nsc_ircc_cb), GFP_KERNEL);
 	if (self == NULL) {
@@ -326,8 +320,8 @@ static int nsc_ircc_open(int i, chipio_t *info)
 	self->tx_buff.head = (__u8 *) kmalloc(self->tx_buff.truesize, 
 					      GFP_KERNEL|GFP_DMA);
 	if (self->tx_buff.head == NULL) {
-		kfree(self);
 		kfree(self->rx_buff.head);
+		kfree(self);
 		return -ENOMEM;
 	}
 	memset(self->tx_buff.head, 0, self->tx_buff.truesize);
@@ -345,8 +339,6 @@ static int nsc_ircc_open(int i, chipio_t *info)
 		ERROR(__FUNCTION__ "(), dev_alloc() failed!\n");
 		return -ENOMEM;
 	}
-	/* dev_alloc doesn't clear the struct, so lets do a little hack */
-	memset(((__u8*)dev)+sizeof(char*),0,sizeof(struct device)-sizeof(char*));
 
 	dev->priv = (void *) self;
 	self->netdev = dev;
@@ -382,6 +374,10 @@ static int nsc_ircc_open(int i, chipio_t *info)
 	self->io.dongle_id = dongle_id;
 	nsc_ircc_init_dongle_interface(self->io.fir_base, dongle_id);
 
+        pmdev = pm_register(PM_SYS_DEV, PM_SYS_IRDA, nsc_ircc_pmproc);
+        if (pmdev)
+                pmdev->data = self;
+
 	return 0;
 }
 
@@ -407,8 +403,6 @@ static int nsc_ircc_close(struct nsc_ircc_cb *self)
 		rtnl_lock();
 		unregister_netdevice(self->netdev);
 		rtnl_unlock();
-		/* Must free the old-style 2.2.x device */
-		kfree(self->netdev);
 	}
 
 	/* Release the PORT that this driver is using */
@@ -608,7 +602,7 @@ static int nsc_ircc_probe_338(nsc_chip_t *chip, chipio_t *info)
 	outb(CFG_PNP0, cfg_base);
 	reg = inb(cfg_base+1);
 	
-	pnp = (reg >> 4) & 0x01;
+	pnp = (reg >> 3) & 0x01;
 	if (pnp) {
 		IRDA_DEBUG(2, "(), Chip is in PnP mode\n");
 		outb(0x46, cfg_base);
@@ -710,8 +704,6 @@ static int nsc_ircc_setup(chipio_t *info)
 		ERROR("%s, Wrong chip version %02x\n", driver_name, version);
 		return -1;
 	}
-	MESSAGE("%s, Found chip at base=0x%03x\n", driver_name, 
-		info->cfg_base);
 
 	/* Switch to advanced mode */
 	switch_bank(iobase, BANK2);
@@ -719,9 +711,12 @@ static int nsc_ircc_setup(chipio_t *info)
 	switch_bank(iobase, BANK0);
 	
 	/* Set FIFO threshold to TX17, RX16, reset and enable FIFO's */
-	switch_bank(iobase, BANK0);	
+	switch_bank(iobase, BANK0);
 	outb(FCR_RXTH|FCR_TXTH|FCR_TXSR|FCR_RXSR|FCR_FIFO_EN, iobase+FCR);
-	
+
+	outb(0x03, iobase+LCR); 	/* 8 bit word length */
+	outb(MCR_SIR, iobase+MCR); 	/* Start at SIR-mode, also clears LSR*/
+
 	/* Set FIFO size to 32 */
 	switch_bank(iobase, BANK2);
 	outb(EXCR2_RFSIZ|EXCR2_TFSIZ, iobase+EXCR2);
@@ -734,11 +729,9 @@ static int nsc_ircc_setup(chipio_t *info)
 	switch_bank(iobase, BANK6);
 	outb(0x20, iobase+0); /* Set 32 bits FIR CRC */
 	outb(0x0a, iobase+1); /* Set MIR pulse width */
-	outb(0x0d, iobase+2); /* Set SIR pulse width */
+	outb(0x0d, iobase+2); /* Set SIR pulse width to 1.6us */
 	outb(0x2a, iobase+4); /* Set beginning frag, and preamble length */
 
-	MESSAGE("%s, driver loaded (Dag Brattli)\n", driver_name);
-
 	/* Enable receive interrupts */
 	switch_bank(iobase, BANK0);
 	outb(IER_RXHDL_IE, iobase+IER);
@@ -815,8 +808,6 @@ static void nsc_ircc_init_dongle_interface (int iobase, int dongle_id)
 			   dongle_types[dongle_id]); 
 		break;
 	case 0x04: /* Sharp RY5HD01 */
-		IRDA_DEBUG(0, __FUNCTION__ "(), %s not supported yet\n",
-			   dongle_types[dongle_id]); 
 		break;
 	case 0x05: /* Reserved, but this is what the Thinkpad reports */
 		IRDA_DEBUG(0, __FUNCTION__ "(), %s not defined by irda yet\n",
@@ -903,8 +894,7 @@ static void nsc_ircc_change_dongle_speed(int iobase, int speed, int dongle_id)
 			   dongle_types[dongle_id]); 
 		break;
 	case 0x04: /* Sharp RY5HD01 */
-		IRDA_DEBUG(0, __FUNCTION__ "(), %s not supported yet\n",
-			   dongle_types[dongle_id]); 
+		break;
 	case 0x05: /* Reserved */
 		IRDA_DEBUG(0, __FUNCTION__ "(), %s not defined by irda yet\n",
 			   dongle_types[dongle_id]); 
@@ -968,7 +958,7 @@ static void nsc_ircc_change_dongle_speed(int iobase, int speed, int dongle_id)
  */
 static void nsc_ircc_change_speed(struct nsc_ircc_cb *self, __u32 speed)
 {
-	struct device *dev = self->netdev;
+	struct net_device *dev = self->netdev;
 	__u8 mcr = MCR_SIR;
 	int iobase; 
 	__u8 bank;
@@ -1045,8 +1035,6 @@ static void nsc_ircc_change_speed(struct nsc_ircc_cb *self, __u32 speed)
 	switch_bank(iobase, BANK2);
 	outb(EXCR2_RFSIZ|EXCR2_TFSIZ, iobase+EXCR2);
 	
-	self->netdev->tbusy = 0;
-	
 	/* Enable some interrupts so we can receive frames */
 	switch_bank(iobase, BANK0); 
 	if (speed > 115200) {
@@ -1062,6 +1050,8 @@ static void nsc_ircc_change_speed(struct nsc_ircc_cb *self, __u32 speed)
     	
 	/* Restore BSR */
 	outb(bank, iobase+BSR);
+	netif_wake_queue(dev);
+	
 }
 
 /*
@@ -1070,12 +1060,12 @@ static void nsc_ircc_change_speed(struct nsc_ircc_cb *self, __u32 speed)
  *    Transmit the frame!
  *
  */
-static int nsc_ircc_hard_xmit_sir(struct sk_buff *skb, struct device *dev)
+static int nsc_ircc_hard_xmit_sir(struct sk_buff *skb, struct net_device *dev)
 {
 	struct nsc_ircc_cb *self;
 	unsigned long flags;
 	int iobase;
-	__u32 speed;
+	__s32 speed;
 	__u8 bank;
 	
 	self = (struct nsc_ircc_cb *) dev->priv;
@@ -1084,14 +1074,20 @@ static int nsc_ircc_hard_xmit_sir(struct sk_buff *skb, struct device *dev)
 
 	iobase = self->io.fir_base;
 
-	/* Lock transmit buffer */
-	if (irda_lock((void *) &dev->tbusy) == FALSE)
-		return -EBUSY;
-	
+	netif_stop_queue(dev);
+		
 	/* Check if we need to change the speed */
-	if ((speed = irda_get_speed(skb)) != self->io.speed)
-		self->new_speed = speed;
-	
+	speed = irda_get_next_speed(skb);
+	if ((speed != self->io.speed) && (speed != -1)) {
+		/* Check for empty frame */
+		if (!skb->len) {
+			nsc_ircc_change_speed(self, speed); 
+			dev_kfree_skb(skb);
+			return 0;
+		} else
+			self->new_speed = speed;
+	}
+
 	spin_lock_irqsave(&self->lock, flags);
 	
 	/* Save current bank */
@@ -1118,25 +1114,31 @@ static int nsc_ircc_hard_xmit_sir(struct sk_buff *skb, struct device *dev)
 	return 0;
 }
 
-static int nsc_ircc_hard_xmit_fir(struct sk_buff *skb, struct device *dev)
+static int nsc_ircc_hard_xmit_fir(struct sk_buff *skb, struct net_device *dev)
 {
 	struct nsc_ircc_cb *self;
 	unsigned long flags;
 	int iobase;
-	__u32 speed;
+	__s32 speed;
 	__u8 bank;
 	int mtt, diff;
 	
 	self = (struct nsc_ircc_cb *) dev->priv;
 	iobase = self->io.fir_base;
 
-	/* Lock transmit buffer */
-	if (irda_lock((void *) &dev->tbusy) == FALSE)
-		return -EBUSY;
-
+	netif_stop_queue(dev);
+	
 	/* Check if we need to change the speed */
-	if ((speed = irda_get_speed(skb)) != self->io.speed)
-		self->new_speed = speed;
+	speed = irda_get_next_speed(skb);
+	if ((speed != self->io.speed) && (speed != -1)) {
+		/* Check for empty frame */
+		if (!skb->len) {
+			nsc_ircc_change_speed(self, speed); 
+			dev_kfree_skb(skb);
+			return 0;
+		} else
+			self->new_speed = speed;
+	}
 
 	spin_lock_irqsave(&self->lock, flags);
 
@@ -1211,7 +1213,7 @@ static int nsc_ircc_hard_xmit_fir(struct sk_buff *skb, struct device *dev)
  out:
 	/* Not busy transmitting anymore if window is not full */
 	if (self->tx_fifo.free < MAX_TX_WINDOW)
-		dev->tbusy = 0;
+		netif_wake_queue(self->netdev);
 
 	/* Restore bank register */
 	outb(bank, iobase+BSR);
@@ -1262,7 +1264,7 @@ static void nsc_ircc_dma_xmit(struct nsc_ircc_cb *self, int iobase)
  * Function nsc_ircc_pio_xmit (self, iobase)
  *
  *    Transmit data using PIO. Returns the number of bytes that actually
- *    got transfered
+ *    got transferred
  *
  */
 static int nsc_ircc_pio_write(int iobase, __u8 *buf, int len, int fifo_size)
@@ -1359,10 +1361,8 @@ static int nsc_ircc_dma_xmit_complete(struct nsc_ircc_cb *self)
 	/* Make sure we have room for more frames */
 	if (self->tx_fifo.free < MAX_TX_WINDOW) {
 		/* Not busy transmitting anymore */
-		self->netdev->tbusy = 0;
-
 		/* Tell the network layer, that we can accept more frames */
-		mark_bh(NET_BH);
+		netif_wake_queue(self->netdev);
 	}
 
 	/* Restore bank */
@@ -1626,11 +1626,9 @@ static void nsc_ircc_sir_interrupt(struct nsc_ircc_cb *self, int eir)
 		if (self->tx_buff.len > 0)
 			self->ier = IER_TXLDL_IE;
 		else { 
-			self->netdev->tbusy = 0; /* Unlock */
-			self->stats.tx_packets++;
-			
-		        mark_bh(NET_BH);	
 
+			self->stats.tx_packets++;
+			netif_wake_queue(self->netdev);
 			self->ier = IER_TXEMP_IE;
 		}
 			
@@ -1737,7 +1735,7 @@ static void nsc_ircc_fir_interrupt(struct nsc_ircc_cb *self, int iobase,
  */
 static void nsc_ircc_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 {
-	struct device *dev = (struct device *) dev_id;
+	struct net_device *dev = (struct net_device *) dev_id;
 	struct nsc_ircc_cb *self;
 	__u8 bsr, eir;
 	int iobase;
@@ -1749,7 +1747,6 @@ static void nsc_ircc_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 	self = (struct nsc_ircc_cb *) dev->priv;
 
 	spin_lock(&self->lock);	
-	dev->interrupt = 1;
 
 	iobase = self->io.fir_base;
 
@@ -1772,7 +1769,6 @@ static void nsc_ircc_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 	outb(self->ier, iobase+IER); /* Restore interrupts */
 	outb(bsr, iobase+BSR);       /* Restore bank register */
 
-	dev->interrupt = 0;
 	spin_unlock(&self->lock);
 }
 
@@ -1818,7 +1814,7 @@ static int nsc_ircc_is_receiving(struct nsc_ircc_cb *self)
  *    Initialize network device
  *
  */
-static int nsc_ircc_net_init(struct device *dev)
+static int nsc_ircc_net_init(struct net_device *dev)
 {
 	IRDA_DEBUG(4, __FUNCTION__ "()\n");
 
@@ -1836,10 +1832,11 @@ static int nsc_ircc_net_init(struct device *dev)
  *    Start the device
  *
  */
-static int nsc_ircc_net_open(struct device *dev)
+static int nsc_ircc_net_open(struct net_device *dev)
 {
 	struct nsc_ircc_cb *self;
 	int iobase;
+	char hwname[32];
 	__u8 bank;
 	
 	IRDA_DEBUG(4, __FUNCTION__ "()\n");
@@ -1863,7 +1860,7 @@ static int nsc_ircc_net_open(struct device *dev)
 	if (request_dma(self->io.dma, dev->name)) {
 		WARNING("%s, unable to allocate dma=%d\n", driver_name, 
 			self->io.dma);
-		free_irq(self->io.irq, self);
+		free_irq(self->io.irq, dev);
 		return -EAGAIN;
 	}
 	
@@ -1878,15 +1875,16 @@ static int nsc_ircc_net_open(struct device *dev)
 	outb(bank, iobase+BSR);
 
 	/* Ready to play! */
-	dev->tbusy = 0;
-	dev->interrupt = 0;
-	dev->start = 1;
+	netif_start_queue(dev);
+	
+	/* Give self a hardware name */
+	sprintf(hwname, "NSC-FIR @ 0x%03x", self->io.fir_base);
 
 	/* 
 	 * Open new IrLAP layer instance, now that everything should be
 	 * initialized properly 
 	 */
-	self->irlap = irlap_open(dev, &self->qos);
+	self->irlap = irlap_open(dev, &self->qos, hwname);
 
 	MOD_INC_USE_COUNT;
 
@@ -1899,7 +1897,7 @@ static int nsc_ircc_net_open(struct device *dev)
  *    Stop the device
  *
  */
-static int nsc_ircc_net_close(struct device *dev)
+static int nsc_ircc_net_close(struct net_device *dev)
 {
 	struct nsc_ircc_cb *self;
 	int iobase;
@@ -1913,9 +1911,8 @@ static int nsc_ircc_net_close(struct device *dev)
 	ASSERT(self != NULL, return 0;);
 
 	/* Stop device */
-	dev->tbusy = 1;
-	dev->start = 0;
-
+	netif_stop_queue(dev);
+	
 	/* Stop and remove instance of IrLAP */
 	if (self->irlap)
 		irlap_close(self->irlap);
@@ -1949,7 +1946,7 @@ static int nsc_ircc_net_close(struct device *dev)
  *    Process IOCTL commands for this device
  *
  */
-static int nsc_ircc_net_ioctl(struct device *dev, struct ifreq *rq, int cmd)
+static int nsc_ircc_net_ioctl(struct net_device *dev, struct ifreq *rq, int cmd)
 {
 	struct if_irda_req *irq = (struct if_irda_req *) rq;
 	struct nsc_ircc_cb *self;
@@ -1967,23 +1964,20 @@ static int nsc_ircc_net_ioctl(struct device *dev, struct ifreq *rq, int cmd)
 	/* Disable interrupts & save flags */
 	save_flags(flags);
 	cli();
-	
+
 	switch (cmd) {
 	case SIOCSBANDWIDTH: /* Set bandwidth */
-		/*
-		 * This function will also be used by IrLAP to change the
-		 * speed, so we still must allow for speed change within
-		 * interrupt context.
-		 */
-		if (!in_interrupt() && !capable(CAP_NET_ADMIN)) {
-			IRDA_DEBUG(0, __FUNCTION__ "(), not capable sysadm\n");
-			return -EPERM;
+		if (!capable(CAP_NET_ADMIN)) {
+			ret = -EPERM;
+			goto out;
 		}
 		nsc_ircc_change_speed(self, irq->ifr_baudrate);
 		break;
 	case SIOCSMEDIABUSY: /* Set media busy */
-		if (!capable(CAP_NET_ADMIN))
-			return -EPERM;
+		if (!capable(CAP_NET_ADMIN)) {
+			ret = -EPERM;
+			goto out;
+		}
 		irda_device_set_media_busy(self->netdev, TRUE);
 		break;
 	case SIOCGRECEIVING: /* Check if we are receiving right now */
@@ -1992,20 +1986,18 @@ static int nsc_ircc_net_ioctl(struct device *dev, struct ifreq *rq, int cmd)
 	default:
 		ret = -EOPNOTSUPP;
 	}
-	
+out:
 	restore_flags(flags);
-	
 	return ret;
 }
 
-static struct net_device_stats *nsc_ircc_net_get_stats(struct device *dev)
+static struct net_device_stats *nsc_ircc_net_get_stats(struct net_device *dev)
 {
 	struct nsc_ircc_cb *self = (struct nsc_ircc_cb *) dev->priv;
 	
 	return &self->stats;
 }
 
-#ifdef CONFIG_APM
 static void nsc_ircc_suspend(struct nsc_ircc_cb *self)
 {
 	MESSAGE("%s, Suspending\n", driver_name);
@@ -2020,19 +2012,10 @@ static void nsc_ircc_suspend(struct nsc_ircc_cb *self)
 
 static void nsc_ircc_wakeup(struct nsc_ircc_cb *self)
 {
-	struct device *dev = self->netdev;
-	int iobase;
-
 	if (!self->io.suspended)
 		return;
 
-	iobase = self->io.fir_base;
-
-	/* Switch to advanced mode */
-	switch_bank(iobase, BANK2);
-	outb(ECR1_EXT_SL, iobase+ECR1);
-	switch_bank(iobase, BANK0);
-
+	nsc_ircc_setup(&self->io);
 	nsc_ircc_net_open(self->netdev);
 	
 	MESSAGE("%s, Waking up\n", driver_name);
@@ -2040,46 +2023,36 @@ static void nsc_ircc_wakeup(struct nsc_ircc_cb *self)
 	self->io.suspended = 0;
 }
 
-static int nsc_ircc_apmproc(apm_event_t event)
+static int nsc_ircc_pmproc(struct pm_dev *dev, pm_request_t rqst, void *data)
 {
-	static int down = 0; /* Filter out double events */
-	int i;
-
-	switch (event) {
-	case APM_SYS_SUSPEND:
-	case APM_USER_SUSPEND:
-		if (!down) {			
-			for (i=0; i<4; i++) {
-				if (dev_self[i])
-					nsc_ircc_suspend(dev_self[i]);
-			}
-		}
-		down = 1;
-		break;
-	case APM_NORMAL_RESUME:
-	case APM_CRITICAL_RESUME:
-		if (down) {
-			for (i=0; i<4; i++) {
-				if (dev_self[i])
-					nsc_ircc_wakeup(dev_self[i]);
-			}
-		}
-		down = 0;
-		break;
-	}
+        struct nsc_ircc_cb *self = (struct nsc_ircc_cb*) dev->data;
+        if (self) {
+                switch (rqst) {
+                case PM_SUSPEND:
+                        nsc_ircc_suspend(self);
+                        break;
+                case PM_RESUME:
+                        nsc_ircc_wakeup(self);
+                        break;
+                }
+        }
 	return 0;
 }
-#endif /* CONFIG_APM */
 
 #ifdef MODULE
 MODULE_AUTHOR("Dag Brattli <dagb@cs.uit.no>");
 MODULE_DESCRIPTION("NSC IrDA Device Driver");
 
 MODULE_PARM(qos_mtt_bits, "i");
+MODULE_PARM_DESC(qos_mtt_bits, "Minimum Turn Time");
 MODULE_PARM(io,  "1-4i");
+MODULE_PARM_DESC(io, "Base I/O addresses");
 MODULE_PARM(irq, "1-4i");
+MODULE_PARM_DESC(irq, "IRQ lines");
 MODULE_PARM(dma, "1-4i");
+MODULE_PARM_DESC(dma, "DMA channels");
 MODULE_PARM(dongle_id, "i");
+MODULE_PARM_DESC(dongle_id, "Type-id of used dongle");
 
 int init_module(void)
 {
